@@ -15,51 +15,53 @@ if not lfs then
     lfs = optionalrequire("lfs")
 end
 
-if not lfs then
-
-    lfs = {
-        getcurrentdir = function()
-            return "."
-        end,
-        attributes = function()
-            return nil
-        end,
-        isfile = function(name)
-            local f = io.open(name,'rb')
-            if f then
-                f:close()
-                return true
-            end
-        end,
-        isdir = function(name)
-            print("you need to load lfs")
-            return false
-        end
-    }
-
-elseif not lfs.isfile then
-
-    local attributes = lfs.attributes
-
-    function lfs.isdir(name)
-        return attributes(name,"mode") == "directory"
-    end
-
-    function lfs.isfile(name)
-        return attributes(name,"mode") == "file"
-    end
-
- -- function lfs.isdir(name)
- --     local a = attributes(name)
- --     return a and a.mode == "directory"
- -- end
-
- -- function lfs.isfile(name)
- --     local a = attributes(name)
- --     return a and a.mode == "file"
- -- end
-
-end
+-- -- see later
+--
+-- if not lfs then
+--
+--     lfs = {
+--         getcurrentdir = function()
+--             return "."
+--         end,
+--         attributes = function()
+--             return nil
+--         end,
+--         isfile = function(name)
+--             local f = io.open(name,'rb')
+--             if f then
+--                 f:close()
+--                 return true
+--             end
+--         end,
+--         isdir = function(name)
+--             print("you need to load lfs")
+--             return false
+--         end
+--     }
+--
+-- elseif not lfs.isfile then
+--
+--     local attributes = lfs.attributes
+--
+--     function lfs.isdir(name)
+--         return attributes(name,"mode") == "directory"
+--     end
+--
+--     function lfs.isfile(name)
+--         return attributes(name,"mode") == "file"
+--     end
+--
+--  -- function lfs.isdir(name)
+--  --     local a = attributes(name)
+--  --     return a and a.mode == "directory"
+--  -- end
+--
+--  -- function lfs.isfile(name)
+--  --     local a = attributes(name)
+--  --     return a and a.mode == "file"
+--  -- end
+--
+-- end
 
 local insert, concat = table.insert, table.concat
 local match, find, gmatch = string.match, string.find, string.gmatch
@@ -72,6 +74,28 @@ local checkedsplit = string.checkedsplit
 
 local P, R, S, C, Cs, Cp, Cc, Ct = lpeg.P, lpeg.R, lpeg.S, lpeg.C, lpeg.Cs, lpeg.Cp, lpeg.Cc, lpeg.Ct
 
+-- better this way:
+
+local tricky     = S("/\\") * P(-1)
+local attributes = lfs.attributes
+
+if sandbox then
+    sandbox.redefine(lfs.isfile,"lfs.isfile")
+    sandbox.redefine(lfs.isdir, "lfs.isdir")
+end
+
+function lfs.isdir(name)
+    if lpegmatch(tricky,name) then
+        return attributes(name,"mode") == "directory"
+    else
+        return attributes(name.."/.","mode") == "directory"
+    end
+end
+
+function lfs.isfile(name)
+    return attributes(name,"mode") == "file"
+end
+
 local colon     = P(":")
 local period    = P(".")
 local periods   = P("..")
@@ -133,8 +157,8 @@ file.suffix       = suffixonly
 file.suffixesonly = suffixesonly
 file.suffixes     = suffixesonly
 
-file.dirname    = pathpart   -- obsolete
-file.extname    = suffixonly -- obsolete
+file.dirname      = pathpart   -- obsolete
+file.extname      = suffixonly -- obsolete
 
 -- actually these are schemes
 
@@ -385,31 +409,90 @@ local deslasher = lpeg.replacer(S("\\/")^1,"/")
 -- then we still have to deal with urls ... anyhow, multiple // are never a real
 -- problem but just ugly.
 
-function file.join(...)
-    local lst = { ... }
-    local one = lst[1]
+-- function file.join(...)
+--     local lst = { ... }
+--     local one = lst[1]
+--     if lpegmatch(isnetwork,one) then
+--         local one = lpegmatch(reslasher,one)
+--         local two = lpegmatch(deslasher,concat(lst,"/",2))
+--         if lpegmatch(hasroot,two) then
+--             return one .. two
+--         else
+--             return one .. "/" .. two
+--         end
+--     elseif lpegmatch(isroot,one) then
+--         local two = lpegmatch(deslasher,concat(lst,"/",2))
+--         if lpegmatch(hasroot,two) then
+--             return two
+--         else
+--             return "/" .. two
+--         end
+--     elseif one == "" then
+--         return lpegmatch(stripper,concat(lst,"/",2))
+--     else
+--         return lpegmatch(deslasher,concat(lst,"/"))
+--     end
+-- end
+
+function file.join(one, two, three, ...)
+    if not two then
+        return one == "" and one or lpegmatch(reslasher,one)
+    end
+    if one == "" then
+        return lpegmatch(stripper,three and concat({ two, three, ... },"/") or two)
+    end
     if lpegmatch(isnetwork,one) then
         local one = lpegmatch(reslasher,one)
-        local two = lpegmatch(deslasher,concat(lst,"/",2))
+        local two = lpegmatch(deslasher,three and concat({ two, three, ... },"/") or two)
         if lpegmatch(hasroot,two) then
             return one .. two
         else
             return one .. "/" .. two
         end
     elseif lpegmatch(isroot,one) then
-        local two = lpegmatch(deslasher,concat(lst,"/",2))
+        local two = lpegmatch(deslasher,three and concat({ two, three, ... },"/") or two)
         if lpegmatch(hasroot,two) then
             return two
         else
             return "/" .. two
         end
-    elseif one == "" then
-        return lpegmatch(stripper,concat(lst,"/",2))
     else
-        return lpegmatch(deslasher,concat(lst,"/"))
+        return lpegmatch(deslasher,concat({  one, two, three, ... },"/"))
     end
 end
 
+-- or we can use this:
+--
+-- function file.join(...)
+--     local n = select("#",...)
+--     local one = select(1,...)
+--     if n == 1 then
+--         return one == "" and one or lpegmatch(stripper,one)
+--     end
+--     if one == "" then
+--         return lpegmatch(stripper,n > 2 and concat({ ... },"/",2) or select(2,...))
+--     end
+--     if lpegmatch(isnetwork,one) then
+--         local one = lpegmatch(reslasher,one)
+--         local two = lpegmatch(deslasher,n > 2 and concat({ ... },"/",2) or select(2,...))
+--         if lpegmatch(hasroot,two) then
+--             return one .. two
+--         else
+--             return one .. "/" .. two
+--         end
+--     elseif lpegmatch(isroot,one) then
+--         local two = lpegmatch(deslasher,n > 2 and concat({ ... },"/",2) or select(2,...))
+--         if lpegmatch(hasroot,two) then
+--             return two
+--         else
+--             return "/" .. two
+--         end
+--     else
+--         return lpegmatch(deslasher,concat({ ... },"/"))
+--     end
+-- end
+
+-- print(file.join("c:/whatever"))
 -- print(file.join("c:/whatever","name"))
 -- print(file.join("//","/y"))
 -- print(file.join("/","/y"))
@@ -495,23 +578,6 @@ function file.collapsepath(str,anchor) -- anchor: false|nil, true, "."
     end
 end
 
--- better this way:
-
-local tricky     = S("/\\") * P(-1)
-local attributes = lfs.attributes
-
-function lfs.isdir(name)
-    if lpegmatch(tricky,name) then
-        return attributes(name,"mode") == "directory"
-    else
-        return attributes(name.."/.","mode") == "directory"
-    end
-end
-
-function lfs.isfile(name)
-    return attributes(name,"mode") == "file"
-end
-
 -- local function test(str,...)
 --    print(string.format("%-20s %-15s %-30s %-20s",str,file.collapsepath(str),file.collapsepath(str,true),file.collapsepath(str,".")))
 -- end
@@ -541,14 +607,17 @@ function file.robustname(str,strict)
     end
 end
 
-file.readdata = io.loaddata
-file.savedata = io.savedata
+local loaddata = io.loaddata
+local savedata = io.savedata
+
+file.readdata  = loaddata
+file.savedata  = savedata
 
 function file.copy(oldname,newname)
     if oldname and newname then
-        local data = io.loaddata(oldname)
+        local data = loaddata(oldname)
         if data and data ~= "" then
-            file.savedata(newname,data)
+            savedata(newname,data)
         end
     end
 end
