@@ -53,6 +53,7 @@ local bittest = bit32.btest
 local rshift = bit32.rshift
 local concat = table.concat
 local lower = string.lower
+local copy = table.copy
 local sub = string.sub
 local strip = string.strip
 local tohash = table.tohash
@@ -69,6 +70,7 @@ local readers           = fonts.handlers.otf.readers
 local streamreader      = readers.streamreader
 
 local setposition       = streamreader.setposition
+local getposition       = streamreader.getposition
 local skipshort         = streamreader.skipshort
 local readushort        = streamreader.readcardinal2  -- 16-bit unsigned integer
 local readulong         = streamreader.readcardinal4  -- 24-bit unsigned integer
@@ -224,10 +226,15 @@ local function readcoverage(f,offset,simple)
     return coverage
 end
 
-local function readclassdef(f,offset)
+local function readclassdef(f,offset,preset)
     setposition(f,offset)
     local classdefformat = readushort(f)
     local classdef = { }
+    if type(preset) == "number" then
+        for k=0,preset-1 do
+            classdef[k] = 1
+        end
+    end
     if classdefformat == 1 then
         local index       = readushort(f)
         local nofclassdef = readushort(f)
@@ -249,6 +256,13 @@ local function readclassdef(f,offset)
     else
         report("unknown classdef format %a ",classdefformat)
     end
+    if type(preset) == "table" then
+        for k in next, preset do
+            if not classdef[k] then
+                classdef[k] = 1
+            end
+        end
+    end
     return classdef
 end
 
@@ -365,7 +379,9 @@ end
 -- We generalize the chained lookups so that we can do with only one handler
 -- when processing them.
 
-local function readlookuparray(f,noflookups)
+-- pruned
+
+local function readlookuparray(f,noflookups,nofcurrent)
     local lookups = { }
     if noflookups > 0 then
         local length = 0
@@ -381,10 +397,34 @@ local function readlookuparray(f,noflookups)
                 lookups[index] = false
             end
         end
+     -- if length > nofcurrent then
+     --     report("more lookups than currently matched characters")
+     -- end
     end
     return lookups
 end
 
+-- not pruned
+--
+-- local function readlookuparray(f,noflookups,nofcurrent)
+--     local lookups = { }
+--     for i=1,nofcurrent do
+--         lookups[i] = false
+--     end
+--     for i=1,noflookups do
+--         local index = readushort(f) + 1
+--         if index > nofcurrent then
+--             report("more lookups than currently matched characters")
+--             for i=nofcurrent+1,index-1 do
+--                 lookups[i] = false
+--             end
+--             nofcurrent = index
+--         end
+--         lookups[index] = readushort(f) + 1
+--     end
+--     return lookups
+-- end
+
 local function unchainedcontext(f,fontdata,lookupid,lookupoffset,offset,glyphs,nofglyphs,what)
     local tableoffset = lookupoffset + offset
     setposition(f,tableoffset)
@@ -409,7 +449,7 @@ local function unchainedcontext(f,fontdata,lookupid,lookupoffset,offset,glyphs,n
                         for i=2,nofcurrent do
                             current[i] = { readushort(f) }
                         end
-                        local lookups = readlookuparray(f,noflookups)
+                        local lookups = readlookuparray(f,noflookups,nofcurrent)
                         rules[#rules+1] = {
                             current = current,
                             lookups = lookups
@@ -433,7 +473,7 @@ local function unchainedcontext(f,fontdata,lookupid,lookupoffset,offset,glyphs,n
         local rules           = { }
         if subclasssets then
             coverage             = readcoverage(f,tableoffset + coverage)
-            currentclassdef      = readclassdef(f,tableoffset + currentclassdef)
+            currentclassdef      = readclassdef(f,tableoffset + currentclassdef,coverage)
             local currentclasses = classtocoverage(currentclassdef,fontdata.glyphs)
             for class=1,#subclasssets do
                 local offset = subclasssets[class]
@@ -452,7 +492,7 @@ local function unchainedcontext(f,fontdata,lookupid,lookupoffset,offset,glyphs,n
                                 for i=2,nofcurrent do
                                     current[i] = currentclasses[readushort(f) + 1]
                                 end
-                                local lookups = readlookuparray(f,noflookups)
+                                local lookups = readlookuparray(f,noflookups,nofcurrent)
                                 rules[#rules+1] = {
                                     current = current,
                                     lookups = lookups
@@ -476,7 +516,7 @@ local function unchainedcontext(f,fontdata,lookupid,lookupoffset,offset,glyphs,n
     elseif subtype == 3 then
         local current    = readarray(f)
         local noflookups = readushort(f)
-        local lookups    = readlookuparray(f,noflookups)
+        local lookups    = readlookuparray(f,noflookups,#current)
         current = readcoveragearray(f,tableoffset,current,true)
         return {
             format = "coverage",
@@ -536,7 +576,7 @@ local function chainedcontext(f,fontdata,lookupid,lookupoffset,offset,glyphs,nof
                             end
                         end
                         local noflookups = readushort(f)
-                        local lookups    = readlookuparray(f,noflookups)
+                        local lookups    = readlookuparray(f,noflookups,nofcurrent)
                         rules[#rules+1] = {
                             before  = before,
                             current = current,
@@ -562,9 +602,9 @@ local function chainedcontext(f,fontdata,lookupid,lookupoffset,offset,glyphs,nof
         local rules           = { }
         if subclasssets then
             local coverage        = readcoverage(f,tableoffset + coverage)
-            local beforeclassdef  = readclassdef(f,tableoffset + beforeclassdef)
-            local currentclassdef = readclassdef(f,tableoffset + currentclassdef)
-            local afterclassdef   = readclassdef(f,tableoffset + afterclassdef)
+            local beforeclassdef  = readclassdef(f,tableoffset + beforeclassdef,nofglyphs)
+            local currentclassdef = readclassdef(f,tableoffset + currentclassdef,coverage)
+            local afterclassdef   = readclassdef(f,tableoffset + afterclassdef,nofglyphs)
             local beforeclasses   = classtocoverage(beforeclassdef,fontdata.glyphs)
             local currentclasses  = classtocoverage(currentclassdef,fontdata.glyphs)
             local afterclasses    = classtocoverage(afterclassdef,fontdata.glyphs)
@@ -604,7 +644,7 @@ local function chainedcontext(f,fontdata,lookupid,lookupoffset,offset,glyphs,nof
                                 end
                                 -- no sequence index here (so why in context as it saves nothing)
                                 local noflookups = readushort(f)
-                                local lookups    = readlookuparray(f,noflookups)
+                                local lookups    = readlookuparray(f,noflookups,nofcurrent)
                                 rules[#rules+1] = {
                                     before  = before,
                                     current = current,
@@ -632,7 +672,7 @@ local function chainedcontext(f,fontdata,lookupid,lookupoffset,offset,glyphs,nof
         local current    = readarray(f)
         local after      = readarray(f)
         local noflookups = readushort(f)
-        local lookups    = readlookuparray(f,noflookups)
+        local lookups    = readlookuparray(f,noflookups,#current)
         before  = readcoveragearray(f,tableoffset,before,true)
         current = readcoveragearray(f,tableoffset,current,true)
         after   = readcoveragearray(f,tableoffset,after,true)
@@ -988,8 +1028,8 @@ function gposhandlers.pair(f,fontdata,lookupid,lookupoffset,offset,glyphs,nofgly
         local nofclasses2  = readushort(f) -- incl class 0
         local classlist    = readpairclasssets(f,nofclasses1,nofclasses2,format1,format2)
               coverage     = readcoverage(f,tableoffset+coverage)
-              classdef1    = readclassdef(f,tableoffset+classdef1)
-              classdef2    = readclassdef(f,tableoffset+classdef2)
+              classdef1    = readclassdef(f,tableoffset+classdef1,coverage)
+              classdef2    = readclassdef(f,tableoffset+classdef2,nofglyphs)
         local usedcoverage = { }
         for g1, c1 in next, classdef1 do
             if coverage[g1] then
@@ -1246,20 +1286,50 @@ do
 
     local plugins = { }
 
-    function plugins.size(f,fontdata,tableoffset,parameters)
-        if not fontdata.designsize then
-            setposition(f,tableoffset+parameters)
-            local designsize = readushort(f)
-            if designsize > 0 then
-                fontdata.designsize    = designsize
-                skipshort(f,2)
-                fontdata.minsize = readushort(f)
-                fontdata.maxsize = readushort(f)
+    function plugins.size(f,fontdata,tableoffset,feature)
+        if fontdata.designsize then
+            -- yes, there are fonts with multiple size entries ... it probably relates
+            -- to the other two fields (menu entries in some language)
+        else
+            local function check(offset)
+                setposition(f,offset)
+                local designsize = readushort(f)
+                if designsize > 0 then -- we could also have a threshold
+                    local fontstyle = readushort(f)
+                    local guimenuid = readushort(f)
+                    local minsize   = readushort(f)
+                    local maxsize   = readushort(f)
+                    if minsize == 0 and maxsize == 0 and fontstyleid == 0 and guimenuid == 0 then
+                        minsize = designsize
+                        maxsize = designsize
+                    end
+                    if designsize >= minsize and designsize <= maxsize then
+                        return minsize, maxsize, designsize
+                    end
+                end
+            end
+            local minsize, maxsize, designsize = check(tableoffset+feature.offset+feature.parameters)
+            if not designsize then
+                -- some old adobe fonts have: tableoffset+feature.parameters and we could
+                -- use some heuristic but why bother ... this extra check will be removed
+                -- some day and/or when we run into an issue
+                minsize, maxsize, designsize = check(tableoffset+feature.parameters)
+                if designsize then
+                    report("bad size feature in %a, falling back to wrong offset",fontdata.filename or "?")
+                else
+                    report("bad size feature in %a,",fontdata.filename or "?")
+                end
+            end
+            if designsize then
+                fontdata.minsize    = minsize
+                fontdata.maxsize    = maxsize
+                fontdata.designsize = designsize
             end
         end
     end
 
-    -- feature order needs checking ... as we loop over a hash
+    -- feature order needs checking ... as we loop over a hash ... however, in the file
+    -- they are sorted so order is not that relevant
 
     local function reorderfeatures(fontdata,scripts,features)
         local scriptlangs  = { }
@@ -1401,7 +1471,7 @@ do
                 feature.parameters = parameters
                 local plugin = plugins[feature.tag]
                 if plugin then
-                    plugin(f,fontdata,offset,parameters)
+                    plugin(f,fontdata,featureoffset,feature)
                 end
             end
         end
@@ -1660,9 +1730,10 @@ do
                                                     flags     = d.flags,
                                                  -- chain     = d.chain,
                                                 }
-                                                sublookuplist[nofsublookups] = h
+                                                sublookuplist[nofsublookups] = copy(h) -- we repack later
                                                 sublookuphash[lookupid] = nofsublookups
                                                 sublookupcheck[lookupid] = 1
+                                                h = nofsublookups --- patch by Hans
                                             else
                                                 report_issue(i,what,sequence,"missing")
                                                 rule.lookups = nil
@@ -1894,7 +1965,7 @@ function readers.gdef(f,fontdata,specification)
                 end
             end
             -- mark sets : todo: just make the same as class sets above
-            if marksetsoffset then
+            if marksetsoffset and marksetsoffset > tableoffset then -- zero offset means no table
                 setposition(f,marksetsoffset)
                 local format = readushort(f)
                 if format == 1 then
@@ -1903,7 +1974,6 @@ function readers.gdef(f,fontdata,specification)
                     for i=1,nofsets do
                         sets[i] = readulong(f)
                     end
-                    -- somehow this fails on e.g. notosansethiopic-bold.ttf
                     for i=1,nofsets do
                         local offset = sets[i]
                         if offset ~= 0 then
