@@ -1,6 +1,6 @@
 -- luatexko.lua
 --
--- Copyright (c) 2013-2017  Dohyun Kim  <nomos at ktug org>
+-- Copyright (c) 2013-2018  Dohyun Kim  <nomos at ktug org>
 --                          Soojin Nam  <jsunam at gmail com>
 --
 -- This work may be distributed and/or modified under the
@@ -13,8 +13,8 @@
 
 luatexbase.provides_module {
   name        = 'luatexko',
-  date        = '2017/05/01',
-  version     = '1.17',
+  date        = '2018/04/19',
+  version     = '1.20',
   description = 'Korean linebreaking and font-switching',
   author      = 'Dohyun Kim, Soojin Nam',
   license     = 'LPPL v1.3+',
@@ -74,6 +74,7 @@ end
 local glue_type_space = 13
 local kern_type_itlc  = 3
 
+local discnode        = node.id("disc")
 local gluenode        = node.id("glue")
 local glyphnode       = node.id("glyph")
 local mathnode        = node.id("math")
@@ -87,6 +88,7 @@ local user_whatsit    = node.subtype("user_defined")
 local n_copy_list     = node.copy_list
 
 local nodedirect        = node.direct
+local d_getdisc         = nodedirect.getdisc
 local d_todirect        = nodedirect.todirect
 local d_tonode          = nodedirect.tonode
 local d_has_field       = nodedirect.has_field
@@ -119,10 +121,11 @@ local d_remove_node     = nodedirect.remove
 local d_nodenew         = nodedirect.new
 local d_nodecount       = nodedirect.count
 local d_end_of_math     = nodedirect.end_of_math
-local d_nodetail        = nodedirect.tail
+local d_nodetail        = nodedirect.slide -- tail seems to be no good
 local d_nodedimensions  = nodedirect.dimensions
 local d_nodefree        = nodedirect.free
 local d_nodewrite       = nodedirect.write
+local d_node_zero_glue  = nodedirect.is_zero_glue
 
 local emsize = 655360
 
@@ -487,7 +490,7 @@ local function is_cjk_k (c)
   or (c >= 0xFE10  and c <= 0xFE1F )
   or (c >= 0xFE30  and c <= 0xFE4F )
   or (c >= 0xFF00  and c <= 0xFFEF )
-  or (c >= 0x20000 and c <= 0x2FA1F)
+  or (c >= 0x20000 and c <= 0x2FFFD)
   or  c == 0x00B0  or  c == 0x2015
   --  or  c == 0x2018  or  c == 0x2019
   --  or  c == 0x201C  or  c == 0x201D
@@ -498,7 +501,7 @@ end
 local function is_hanja (c)
   return (c >= 0x3400 and c <= 0x9FFF )
   or (c >= 0xF900 and c <= 0xFAFF )
-  or (c >= 0x20000 and c <= 0x2FA1F)
+  or (c >= 0x20000 and c <= 0x2FFFD)
 end
 
 local function is_hangul (c)
@@ -613,28 +616,28 @@ local function d_get_hlist_char_first (hlist)
       local c,f = d_get_hlist_char_first(curr)
       if c then return c,f end
     elseif currid == gluenode then
-      if d_getglue(curr) ~= 0 then return end
+      if not d_node_zero_glue(curr) then return end
     end
     curr = d_getnext(curr)
   end
 end
 
-local function d_get_hlist_char_last (hlist,prevchar,prevfont)
+local function d_get_hlist_char_last (hlist,c,f)
   local curr = d_nodetail(d_getlist(hlist))
   while curr do
     local currid = d_getid(curr)
     if currid == glyphnode then
-      local c,f = d_get_unicode_char(curr), d_getfont(curr)
-      if c and not is_unicode_vs(c) then return c,f end -- bypass VS
+      c, f = d_get_unicode_char(curr), d_getfont(curr)
+      if c and not is_unicode_vs(c) then break end -- bypass VS
     elseif currid == hlistnode or currid == vlistnode then
-      local c,f = d_get_hlist_char_last(curr)
-      if c then return c,f end
+      c, f = d_get_hlist_char_last(curr)
+      if c then break end
     elseif currid == gluenode then
-      if d_getglue(curr) ~= 0 then return end
+      if not d_node_zero_glue(curr) then break end
     end
     curr = d_getprev(curr)
   end
-  return prevchar, prevfont
+  return c, f
 end
 
 ----------------------------
@@ -1500,9 +1503,7 @@ local function font_substitute(head)
   local curr = head
   while curr do
     local currid = d_getid(curr)
-    if currid == mathnode and d_getsubtype(curr) == 0 then
-      curr = d_end_of_math(curr)
-    elseif currid == glyphnode then
+    if currid == glyphnode then
       local currchar, currfont = d_getchar(curr), d_getfont(curr)
       if currchar then
         if is_unicode_vs(currchar) then -- ideograph variation selectors
@@ -1588,6 +1589,15 @@ local function font_substitute(head)
           end
         end
       end
+    elseif currid == mathnode then
+      if d_getsubtype(curr) == 0 then
+        curr = d_end_of_math(curr)
+      end
+    elseif currid == discnode then
+      local pre, post, replace = d_getdisc(curr)
+      if pre     then font_substitute(pre)     end
+      if post    then font_substitute(post)    end
+      if replace then font_substitute(replace) end
     end
     curr = d_getnext(curr)
   end
