@@ -1,21 +1,14 @@
-# $Id: TLTREE.pm 42428 2016-11-03 02:58:32Z preining $
+# $Id: TLTREE.pm 48145 2018-07-05 21:20:15Z karl $
 # TeXLive::TLTREE.pm - work with the tree of all files
-# Copyright 2007-2016 Norbert Preining
+# Copyright 2007-2018 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 
 package TeXLive::TLTREE;
 
-my $svnrev = '$Revision: 42428 $';
-my $_modulerevision;
-if ($svnrev =~ m/: ([0-9]+) /) {
-  $_modulerevision = $1;
-} else {
-  $_modulerevision = "unknown";
-}
-sub module_revision {
-  return $_modulerevision;
-}
+my $svnrev = '$Revision: 48145 $';
+my $_modulerevision = ($svnrev =~ m/: ([0-9]+) /) ? $1 : "unknown";
+sub module_revision { return $_modulerevision; }
 
 use TeXLive::TLUtils;
 
@@ -72,6 +65,119 @@ sub init_from_files {
   $self->_initialize_lines(@lines);
 }
 
+
+sub init_from_git {
+  my $self = shift;
+  my $svnroot = $self->{'svnroot'};
+  my $retval = $?;
+  my %files;
+  my %deletedfiles;
+  my @lines;
+
+  my @foo = `cd $svnroot; git log --pretty=format:COMMIT=%h --no-renames --name-status`;
+  if ($retval != 0) {
+    $retval /= 256 if $retval > 0;
+    tldie("TLTree: git log in $svnroot returned $retval, stopping.\n");
+  }
+  chomp(@foo);
+
+  my $curcom = "";
+  for my $l (@foo) {
+    if ($l eq "") {
+      $curcom = "";
+      next;
+    } elsif ($l =~ m/^COMMIT=([[:xdigit:]]*)$/) {
+      $curcom = $1;
+      $rev++;
+      next;
+    } else {
+      # output is 
+      #   STATUS FILENAME
+      # where STATUS is as follows:
+      #   Added (A), Copied (C), Deleted (D), Modified (M), Renamed (R), have their type (i.e. regular file,
+      #   symlink, submodule, ...) changed (T), are Unmerged (U), are Unknown (X), or have had their pairing Broken (B).
+      if ($l =~ m/^(A|C|D|M|R|T|U|X|B)\S*\s+(.*)$/) {
+        my $status = $1;
+        my $curfile = $2;
+        #
+        # check whether the file was already removed
+        if (!defined($files{$curfile}) && !defined($deletedfiles{$curfile})) {
+          # first occurrence of that file
+          if ($status eq "D") {
+            $deletedfiles{$curfile} = 1;
+          } else {
+            $files{$curfile} = $rev;
+          }
+        }
+      } else {
+        print STDERR "Unknown line in git output: >>$l<<\n";
+      }
+    }
+  }
+
+  # now reverse the order
+  for my $f (keys %files) {
+    my $n = - ( $files{$f} - $rev ) + 1;
+    push @lines, "             $n $n dummy $f"
+  }
+  # TODO needs to be made better!
+  $self->{'revision'} = $rev;
+  $self->_initialize_lines(@lines);
+}
+
+sub init_from_gitsvn {
+  my $self = shift;
+  my $svnroot = $self->{'svnroot'};
+  my @foo = `cd $svnroot; git log --pretty=format:%h --name-only`;
+  chomp(@foo);
+  my $retval = $?;
+  if ($retval != 0) {
+    $retval /= 256 if $retval > 0;
+    tldie("TLTree: git log in $svnroot returned $retval, stopping.\n");
+  }
+  my %com2rev;
+  my @lines;
+  my $curcom = "";
+  my $currev = "";
+  for my $l (@foo) {
+    if ($l eq "") {
+      $currev = "";
+      $curcom = "";
+      next;
+    }
+    if ($curcom eq "") {
+      # now we should get a commit!
+      # we could also pattern match on 8 hex digits, but that costs time!
+      $curcom = $l;
+      $currev = `git svn find-rev $curcom`;
+      chomp($currev);
+      if (!$currev) {
+        # found a commit without svn rev, try to find it under the parents
+        my $foo = $curcom;
+        my $nr = 0;
+        while (1) {
+          $foo .= "^";
+          $nr++;
+          my $tr = `git svn find-rev $foo`;
+          chomp($tr);
+          if ($tr) {
+            # we add the number of parents to the currev
+            $currev = $tr + $nr;
+            last;
+          }
+        }
+      }
+      $com2rev{$curcom} = $currev;
+    } else {
+      # we got a file name
+      push @lines, "             $currev $currev dummy $l"
+    }
+  }
+  # TODO needs to be made better!
+  $self->{'revision'} = 1;
+  $self->_initialize_lines(@lines);
+}
+
 sub _initialize_lines {
   my $self = shift;
   my @lines = @_;
@@ -213,13 +319,21 @@ of bin patterns for that arch).
 sub get_matching_files {
   my ($self, $type, $p, $pkg, $arch) = @_;
   my $ARCH = $arch;
-  my $PKGNAME = $pkg;
   my $newp;
-  eval "\$newp = \"$p\"";
-  if (!defined($newp)) {
-    print "Huuu: cannot generate newp from p: p=$p, pkg=$pkg, arch=$arch, type=$type\n";
+  {
+    my $warnstr = "";
+    local $SIG{__WARN__} = sub { $warnstr = $_[0]; };
+    eval "\$newp = \"$p\"";
+    if (!defined($newp)) {
+      die "cannot set newp from p: p=$p, pkg=$pkg, arch=$arch, type=$type";
+    }
+    if ($warnstr) {
+      tlwarn("Warning `$warnstr' while evaluating: $p "
+             . "(pkg=$pkg, arch=$arch, type=$type), returning empty list\n");
+      return ();
+    }
   }
-  return($self->_get_matching_files($type,$newp));
+  return $self->_get_matching_files($type,$newp);
 }
 
   
@@ -236,7 +350,7 @@ sub _get_matching_files {
   } elsif ($pattype eq "d") {
     @matchfiles = $self->files_under_path($patdata);
   } else {
-    die "Unknown pattern pattern type `$pattype' in $p";
+    die "Unknown pattern type `$pattype' in $p";
   }
   ddebug("p=$p; matchfiles=@matchfiles\n");
   return @matchfiles;
